/*
Copyright (C) 2021 Hygon Info Technologies Ltd.
      All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are not permitted without the prior consent of Hygon Info
Technologies.
*/

#ifndef _TKM_API_H_
#define _TKM_API_H_

#include <stdint.h>

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility ("default")))
#define LOCAL_API  __attribute__ ((visibility ("hidden")))
#endif

typedef unsigned long phy_addr_t;
#define MAX_BATCH_NUMS      128
#define SM2_KEY_SIZE        32
#define SM3_HASH_SIZE       32
#define SM3_BLOCK_SIZE      64
#define SM4_KEY_LEN         16
#define SM4_BLOCK_SIZE      16
#define SM4_IV_LEN          SM4_BLOCK_SIZE
#define AES_128_KEY_LEN     16
#define AES_192_KEY_LEN     24
#define AES_256_KEY_LEN     32
#define DES_192_KEY_LEN     24
#define RSA_LEN_NUM         5
#define RSA_1024_MOD_LEN    128
#define RSA_2048_MOD_LEN    256
#define RSA_4096_MOD_LEN    512
#define RSA_1024_KEY_LEN    (RSA_1024_MOD_LEN * RSA_LEN_NUM)
#define RSA_2048_KEY_LEN    (RSA_2048_MOD_LEN * RSA_LEN_NUM)
#define RSA_4096_KEY_LEN    (RSA_4096_MOD_LEN * RSA_LEN_NUM)
#define EC_192_KEY_LEN      24
#define EC_239_KEY_LEN      30
#define EC_256_KEY_LEN      32
#define ED25519_256_KEY_LEN 32
#define ECIES_256_KEY_LEN   32
#define MAX_HASH_LEN        64
#define AES_IV_LEN          16
#define DES_IV_LEN          8
#define AES_BLOCK_SIZE      16
#define DES_BLOCK_SIZE      8
#define MAX_SUPPORT_KEY_LEN RSA_4096_KEY_LEN
#define MAX_UID_LEN         32

// Sharedkey gen role
#define ROLE_REQUESTOR      (0)
#define ROLE_RESPONSER      (1)

#define FLAG_CRYPT_TYPE_DEC      0x1
#define FLAG_KEYGEN_LOAD         0x2
#define FLAG_KEYGEN_USER_INPUT   0x4
#define FLAG_SHAREDKEY_S1_SA_GEN 0x8
#define FLAG_KEY_EXPORTABLE      0x10000
#define FLAG_KEY_ENC_ONLY        0x20000

// Padding mode
#define PKCS7_PADDING_MODE  0x2
#define PKCS5_PADDING_MODE  0x3
#define NONE_PADDING_MODE   0x4

#define ISK_SM2_ENC_TYPE 0x1
#define ISK_SM2_SIGN_TYPE 0x2

// Get key_type and index from handle
#define TKM_KEY_TYPE(handle)                  ((uint32_t)(handle) >> 24)
#define TKM_KEY_INDEX(handle)                 ((uint32_t)(handle) & ((1 << 24) - 1))
// Generate handle by key_type and index
#define TKM_GEN_HANDLE(key_type, index)       (((uint32_t)(key_type) << 24) | (((uint32_t)(index)) & ((1 << 24) - 1)))

// Log defination
#ifndef PRT
#define PRT(fmt, ...) \
    printf(fmt, ##__VA_ARGS__);
#define ERR_PRT(fmt, ...) \
    printf("ERR: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define WARN_PRT(fmt, ...) \
    printf("WARN: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define INFO_PRT(fmt, ...) \
    printf("INFO: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define DBG_PRT(fmt, ...) \
    do {                                            \
        if (__glibc_unlikely(psp_sys_dbg_is_open()))    \
            printf("DBG: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__);  \
    } while (0)
#endif

#define ERR_COND(cond, err, label, fmt, ...) \
    if (cond) {                              \
        ERR_PRT(fmt, ##__VA_ARGS__);         \
        ret = err;                           \
        goto label;                          \
    }

#define TKM_BATCH_RESULT_CHECK(reqs, batch, label, fmt, ...) \
    do {                                                   \
        uint8_t bn = 0; \
        for (bn = 0; bn < batch; bn++) { \
            if ((reqs)[bn].return_code) { \
                ret = -TKM_ERR_BATCH; \
                ERR_PRT("return_code[%d]: %d[%s] for " fmt, bn, (reqs)[bn].return_code, TKM_ERR_STR((reqs)[bn].return_code), ##__VA_ARGS__); \
                goto label; \
            } \
        } \
    } while (0)

#define TKM_ERR_STR(x) (get_tkm_rc_str(-(x)))
EXPORT_API const char *get_tkm_rc_str(uint32_t rc);
enum TKM_ERROR_TYPE {
    TKM_NORMAL = 0x0,
    TKM_ERR_VERSION,
    TKM_NULL_PTR,
    TKM_ERR_ADDR_MAPPING,
    TKM_EXT_BUF_INITED,
    TKM_ERR_SIZE_SMALL,
    TKM_ERR_BEYOND_MAX,
    TKM_ERR_STATUS,
    TKM_EXT_BUF_DAMAGED,
    TKM_KEY_FILE_DAMAGED,
    TKM_ERR_MEM,
    TKM_ERR_API,
    TKM_ERR_ALGO,
    TKM_ERR_SIZE,
    TKM_ERR_TYPE,
    TKM_ERR_KEY_NOT_EXIST,
    TKM_ERR_SLOT_NOT_ALLOC,
    TKM_UNSUPPORTED_CMD,
    TKM_UNSUPPORTED_SUBCMD,
    TKM_ERR_ISK0_UNCHANGEABLE,
    TKM_ERR_SN,
    TKM_ERR_CCP_VQ_STATUS,
    TKM_ERR_CCP_SM2,
    TKM_ERR_CCP_SM4,
    TKM_ERR_INTEGRITY,
    TKM_ERR_PADDING_MODE,
    TKM_ERR_SM2_ENCRYPT,
    TKM_UNSUPPORTED_APP_KEY,
    TKM_ERR_SM2_DECRYPT,
    TKM_ERR_CERT,
    TKM_ERR_TIME,
    TKM_INVALID_IMPORT_KEY,
    TKM_ERR_DEV_CERT_TYPE,
    TKM_INVALID_KEY_FMT,
    TKM_ERR_CCP_SM3,
    TKM_ERR_KEY_EXIST,
    TKM_ERR_AUTH_TYPE,
    TKM_ERR_AUTH_CODE,
    TKM_INVALID_FLAG,
    TKM_DA_PROTECT_PERIOD,
    TKM_UNSUPPORTED_IDX0,
    TKM_FLASH_NOT_INIT,
    TKM_ERR_BATCH,
    TKM_UNSUPPORTED_ATTR,
    TKM_ERR_GEN_SHARED_KEY,
    TKM_UNSUPPORTED_SCHEME,
    TKM_UNSUPPORTED_IMAGE,
    TKM_UNSUPPORTED_DIE,
    TKM_ERR_CRYPT_MODE,
    TKM_ERR_KDF_MODE,
    TKM_UDRIVER_START = 0x1000,
    TKM_ERR_INIT,
    TKM_ERR_ROLE,
    TKM_UDRIVER_END,
    TKM_MAX_ERR_TYPE
};

/*********************************************************************/
#ifndef __SGD__
#define __SGD__
// GMT 0006
#define SGD_SM4_GCM 0x02000400
#define SGD_SM4_CCM 0x04000400
/* block cipher modes */
#define SGD_ECB 0x01
#define SGD_CBC 0x02
#define SGD_CFB 0x04
#define SGD_OFB 0x08
#define SGD_MAC 0x10
#define SGD_CTR 0x20
/* stream cipher modes */
#define SGD_EEA3 0x01
#define SGD_EIA3 0x02
/* ciphers */
#define SGD_SM1 0x00000100
#define SGD_SSF33 0x00000200
#define SGD_SM4 0x00000400
#define SGD_ZUC 0x00000800
#define SGD_AES 0x10000400
#define SGD_3DES 0x10000800
/* ciphers with modes */
#define SGD_SM1_ECB (SGD_SM1|SGD_ECB)
#define SGD_SM1_CBC (SGD_SM1|SGD_CBC)
#define SGD_SM1_CFB (SGD_SM1|SGD_CFB)
#define SGD_SM1_OFB (SGD_SM1|SGD_OFB)
#define SGD_SM1_CTR (SGD_SM1|SGD_CTR)
#define SGD_SM1_MAC (SGD_SM1|SGD_MAC)
#define SGD_SSF33_ECB (SGD_SSF33|SGD_ECB)
#define SGD_SSF33_CBC (SGD_SSF33|SGD_CBC)
#define SGD_SSF33_CFB (SGD_SSF33|SGD_CFB)
#define SGD_SSF33_OFB (SGD_SSF33|SGD_OFB)
#define SGD_SSF33_MAC (SGD_SSF33|SGD_MAC)
#define SGD_SM4_ECB (SGD_SM4|SGD_ECB)
#define SGD_SM4_CBC (SGD_SM4|SGD_CBC)
#define SGD_SM4_CFB (SGD_SM4|SGD_CFB)
#define SGD_SM4_OFB (SGD_SM4|SGD_OFB)
#define SGD_SM4_MAC (SGD_SM4|SGD_MAC)
#define SGD_SM4_CTR (SGD_SM4|SGD_CTR)
#define SGD_ZUC_EEA3 (SGD_ZUC|SGD_EEA3)
#define SGD_ZUC_EIA3 (SGD_ZUC|SGD_EIA3)
#define SGD_AES_ECB (SGD_AES|SGD_ECB)
#define SGD_AES_CBC (SGD_AES|SGD_CBC)
#define SGD_AES_CFB (SGD_AES|SGD_CFB)
#define SGD_AES_OFB (SGD_AES|SGD_OFB)
#define SGD_AES_CTR (SGD_AES|SGD_CTR) // add later
#define SGD_AES_MAC (SGD_AES|SGD_MAC)
#define SGD_3DES_ECB (SGD_3DES|SGD_ECB)
#define SGD_3DES_CBC (SGD_3DES|SGD_CBC)
#define SGD_3DES_CFB (SGD_3DES|SGD_CFB)
#define SGD_3DES_OFB (SGD_3DES|SGD_OFB)
/* public key usage */
#define SGD_PK_SIGN 0x0100
#define SGD_PK_DH 0x0200
#define SGD_PK_ENC 0x0400
/* public key types */
#define SGD_RSA 0x00010000
#define SGD_RSA_SIGN (SGD_RSA|SGD_PK_SIGN)
#define SGD_RSA_ENC (SGD_RSA|SGD_PK_ENC)
#define SGD_RSA_1 0x00010200
#define SGD_RSA_3 0x00010800
#define SGD_SM2 0x00020100   //SM2
#define SGD_SM2_1 0x00020200 //SM2 sign
#define SGD_SM2_2 0x00020400 //SM2 key exchange
#define SGD_SM2_3 0x00020800 //SM2 encrypt
#define SGD_SHA224 0x00000008
#define SGD_SHA384 0x00000010
#define SGD_SHA512 0x00000020
#define SGD_SYM 0x11000401
/* hash */
#define SGD_SM3 0x00000001
#define SGD_SHA1 0x00000002
#define SGD_SHA256 0x00000004
/* hmac */
#define SGD_SHA1_HMAC 0x00010012
#define SGD_SHA256_HMAC 0x00010014
#define SGD_SM3_HMAC 0x00020211

#define KEY_CIPHER_LEN 3072
#define KEY_HASH_LEN 32
//#define MAX_SYMKEY_LEN 496
#define MAX_SYMKEY_LEN 32
#define SHA1_DIGEST_LENGTH   20
#define SHA256_DIGEST_LENGTH 32
#define SM3_DIGEST_LENGTH    32

#define RSAref_MAX_BITS         4096
#define RSAref_MAX_LEN          ((RSAref_MAX_BITS + 7) / 8)
#define RSAref_MAX_PBITS        ((RSAref_MAX_BITS + 1) / 2)
#define RSAref_MAX_PLEN         ((RSAref_MAX_PBITS + 7)/ 8)
#define ECCref_MAX_BITS         512
#define ECCref_MAX_LEN          ((ECCref_MAX_BITS+7) / 8)

typedef struct ECCrefPublicKey_st { /* ECC */
    unsigned int bits; /* */
    unsigned char x[ECCref_MAX_LEN]; /* x */
    unsigned char y[ECCref_MAX_LEN]; /* y */
} ECCrefPublicKey;
typedef ECCrefPublicKey ecc_ref_public_key;

typedef struct RSArefPublicKey_st {
    unsigned int bits;
    unsigned char m[RSAref_MAX_LEN];
    unsigned char e[RSAref_MAX_LEN];
} RSArefPublicKey;
typedef RSArefPublicKey rsa_ref_public_key;

typedef struct ECCCipher_st {
    unsigned char x[ECCref_MAX_LEN];
    unsigned char y[ECCref_MAX_LEN];
    unsigned char M[32];
    unsigned int L;
    unsigned char C[1]; // GMT0018
} ECCCipher;
typedef ECCCipher ecc_cipher;

typedef struct ECCSignature_st {
    unsigned char r[ECCref_MAX_LEN];
    unsigned char s[ECCref_MAX_LEN];
} ECCSignature;
typedef ECCSignature ecc_signature;
#endif

typedef union {
    ecc_ref_public_key ecc_pub;
    rsa_ref_public_key rsa_pub;
} asym_pub_key_t;

/*********************************************************************/
// generic crypto struct
#define IV_MAX_SIZE 32
struct sym_alg_para {
    unsigned int alg_id; // default: SM4
    unsigned int key_bits; // default: 128 bits
    unsigned int mode; // default: ECB
    unsigned int padding_mode; // default:nopadding
    unsigned int hash_alg; // check integrity
    unsigned char iv[IV_MAX_SIZE];
};

#define TKM_ECC_SM2_P256  0x1
struct ecc_alg_para {
    unsigned int curve_id; //defaut: sm2
    unsigned int scheme; // uiAsymKeyType: sm2 ecdsa
    unsigned int hash_alg;
};

#define RSA_PKCS1_PADDING       1
#define RSA_SSLV23_PADDING      2
#define RSA_NO_PADDING          3
#define RSA_PKCS1_OAEP_PADDING  4
#define RSA_X931_PADDING        5
struct rsa_alg_para {
    unsigned int scheme; // default: pkcs#1.5
    unsigned int key_bits;
    unsigned int exponent;
    unsigned int hash_alg;
};

struct keyedhash_alg_para {
    unsigned int hash_alg;
};

typedef union {
    struct sym_alg_para sap; //sym cmac
    struct rsa_alg_para rap;
    struct ecc_alg_para eap;
    struct keyedhash_alg_para kap;
} public_parms;

#define TKM_ALG_NULL 0x0
#define TKM_ALG_RSA 0x1
#define TKM_ALG_ECC 0x2
#define TKM_ALG_SYMCIPHER 0x3
#define TKM_ALG_KEYEDHASH 0x4
#define TKM_ALG_CMAC 0x5
typedef struct alg_public_para {
    unsigned int alg_type; // 0:default, RSA, ECC, SYMCIPHER, HMAC, CMAC
    public_parms para;
} alg_pub_para_t;

/********************************************************************/
enum SYS_SEC_ROLE {
    SYS_DEV_ADMIN = 0,
    SYS_KEY_ADMIN,
    SYS_ROLE_MAX
};
// authcode
#define AUTH_VALUE_LEN_MAX      32
#define DEFAULT_AUTH_LEN        4
#define AUTH_FLAG_SYS           0x100

enum AUTH_TYPE {
    AUTH_TYPE_NO_AUTH = 0,
    AUTH_TYPE_PW4B,
    AUTH_TYPE_NR_MAX
};

struct tkm_auth_info {
    uint32_t auth_flag; // bit31~9: reserved, bit8: sys_auth, bit7~0: auth_type from AUTH_TYPE
    uint8_t auth_val[AUTH_VALUE_LEN_MAX];
};

// device configuration
enum DEV_CONFIG_TYPE {
    CONFIG_TPV_EN = 0,
    CONFIG_KEY_HASH_FCHK_EN,
    CONFIG_USER_PUBKEY,
    DEV_CONFIG_MAX
};

// key protection scheme
enum KEY_PROT_SCHEME {
    /* default scheme, only support to backup and restore on the local device */
    KEY_PROT_DEFAULT = 0,
    /* support to backup and restore on the same generation device */
    KEY_PROT_SAME_GENERATION,
    /* support to backup and restore by device certification */
    KEY_PROT_DEV_KEY_CERT,
    KEY_PROT_MAX
};

// kdf mode
enum KDF_MODE {
    HMAC_MODE = 0,
    KDF_MODE_MAX
};

// Key type defination
#define ANONYMOUS_KEY_TYPE 0xff
enum KEY_ALG_TYPE {
    ALG_UNSPECIFIED = 0,
    ALG_SYM_KEY,
    ALG_SM4,
    ALG_AES,
    ALG_AES_128,
    ALG_AES_192,
    ALG_AES_256,
    ALG_3DES,
    ALG_ASYM_KEY,
    ALG_SM2,
    ALG_SM2_SIGN,
    ALG_SM2_ENC,
    ALG_RSA,
    ALG_RSA_SIGN,
    ALG_RSA_ENC,
    ALG_RSA_1024,
    ALG_RSA_2048,
    ALG_RSA_4096,
    ALG_EC,
    ALG_EC_192,
    ALG_EC_239,
    ALG_EC_256,
    ALG_ED25519_256,
    ALG_ECIES_256,
    ALG_RSA_1024_SIGN,
    ALG_RSA_1024_ENC,
    ALG_RSA_2048_SIGN,
    ALG_RSA_2048_ENC,
    ALG_RSA_4096_SIGN,
    ALG_RSA_4096_ENC,
    ALG_MAX
};
// internal key
#define INTERNAL_KEY_START      0
#define KEK_SM4                 ALG_SM4
#define ISK_SM2_SIGN            ALG_SM2_SIGN
#define ISK_SM2_ENCRYPT         ALG_SM2_ENC
#define ISK_RSA_2048_SIGN       ALG_RSA_2048_SIGN
#define ISK_RSA_2048_ENCRYPT    ALG_RSA_2048_ENC
#define INTERNAL_KEY_END        ALG_MAX

// application key
#define APP_KEY_START       (0x80 | 0)
#define APP_KEY_SM4         (0x80 | ALG_SM4)
#define APP_KEY_SM2         (0x80 | ALG_SM2)
#define APP_KEY_AES_128     (0x80 | ALG_AES_128)
#define APP_KEY_AES_192     (0x80 | ALG_AES_192)
#define APP_KEY_AES_256     (0x80 | ALG_AES_256)
#define APP_KEY_DES_192     (0x80 | ALG_3DES)
#define APP_KEY_RSA_1024    (0x80 | ALG_RSA_1024)
#define APP_KEY_RSA_2048    (0x80 | ALG_RSA_2048)
#define APP_KEY_RSA_4096    (0x80 | ALG_RSA_4096)
#define APP_KEY_EC_192      (0x80 | ALG_EC_192)
#define APP_KEY_EC_239      (0x80 | ALG_EC_239)
#define APP_KEY_EC_256      (0x80 | ALG_EC_256)
#define APP_KEY_ED25519_256 (0x80 | ALG_ED25519_256)
#define APP_KEY_ECIES_256   (0x80 | ALG_ECIES_256)
#define APP_KEY_END         (0x80 | ALG_MAX)

#define KEY_ECC_PRI_KEY_FMT     0
#define KEY_ECC_PUB_PRI_FMT     1
#define KEY_CIPHERTEXT          0
#define KEY_PLAINTEXT           1

union key_type {
    uint32_t data;
    struct {
        uint32_t ktype           : 8; // references for enum KEY_ALG_TYPE
        uint32_t kfmt            : 2; // ECC: 0: default format private key, 1: 04|x|y|pri_key
        uint32_t weak_hash       : 1; // 0: hash=HASH(type | len | plainkey), 1: hash=HASH(plainkey)
        uint32_t rsvd1           : 5; // Reserved
        uint32_t kattr_exportable: 1;
        uint32_t kattr_ke_only   : 1;
        uint32_t kattr_rsvd      : 14;
    } fields;
};

struct key_cipher {
    union key_type type;
    uint32_t len;
    uint8_t buf[KEY_CIPHER_LEN];
    uint8_t hash[KEY_HASH_LEN];
};

struct tkm_device_info {
    uint32_t api_version;
    uint32_t fw_version;
    uint32_t kek_sm4_total;
    uint32_t isk_sm2_sign_total;
    uint32_t isk_sm2_enc_total;
    uint8_t chip_id[32];
} __attribute__((packed));

struct key_info_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key_idx;
    uint32_t bitmap_info;
} __attribute__((packed));

struct key_gen_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key_idx;
    struct tkm_auth_info auth;
} __attribute__((packed));

struct key_refresh_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key_idx;
    struct tkm_auth_info auth;
} __attribute__((packed));

struct change_keyauth_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info old_auth;
    struct tkm_auth_info new_auth;
} __attribute__((packed));

struct ecc_sign_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
    uint8_t *digest;
    uint32_t dgst_len;
    ecc_signature *sig;
} __attribute__((packed));

struct ecc_decrypt_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
    ecc_cipher *enc_data;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

struct ecc_pubkey_export_request {
    int32_t return_code;
    uint32_t handle;
    ecc_ref_public_key *ecc_pub;
} __attribute__((packed));

struct rsa_pubkey_export_request {
    int32_t return_code;
    uint32_t handle;
    rsa_ref_public_key *rsa_pub;
} __attribute__((packed));

struct hmac_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key;
    uint32_t alg_id;
    struct tkm_auth_info auth;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

typedef struct hmac_kdf {
    uint32_t halg;
    uint32_t hmac_key;
    struct tkm_auth_info auth;
    uint8_t *src_data;
    uint32_t src_len;
} __attribute__((packed)) hmac_kdf_t;

typedef union {
    hmac_kdf_t hmac;
} kdf_param_t;

struct kdf_request {
    int return_code;
    uint32_t flag;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t sym_padding;
    uint32_t alg_id;
    uint32_t key_bits;
    uint32_t kdf_mode;
    kdf_param_t kdf_para;
    uint32_t handle;
    struct tkm_auth_info auth_setting;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct sym_keygen_sym_encrypt_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t handle;
    struct tkm_auth_info auth_setting;
    uint32_t padding_mode;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct keyimport_decrypt_and_load_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t key2;
    struct tkm_auth_info auth2;
    uint32_t padding_mode;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct key_destroy_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
} __attribute__((packed));

struct sym_crypt_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t padding_mode;
    uint32_t crypt_mode;
    uint8_t *iv;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

struct asym_keygen_sym_request {
    int32_t return_code;
    uint32_t flag; 
    uint32_t key;
    alg_pub_para_t alg_param;
    struct tkm_auth_info auth;
    uint32_t sym_padding;
    uint32_t handle;
    struct tkm_auth_info auth_setting;
    struct key_cipher *pri_key;
    asym_pub_key_t *apk;
} __attribute__((packed));

struct rsa_sign_request {
    int32_t return_code;
    uint32_t key;
    uint32_t hash_alg;
    uint32_t padding; // default: RSA_PKCS1_PADDING
    int32_t salt_len;
    struct tkm_auth_info auth;
    uint8_t *src;
    uint32_t src_len;
    uint8_t *sig_data;
    uint32_t sig_data_len;
} __attribute__((packed));

struct rsa_decrypt_request {
    int32_t return_code;
    uint32_t key;
    uint32_t padding; // default: RSA_PKCS1_PADDING
    struct tkm_auth_info auth;
    uint8_t *src;
    uint32_t src_len;
    uint8_t *dest;
    uint32_t dest_len;
} __attribute__((packed));

struct cmac_request {
    int32_t return_code;
    uint32_t key;
    uint32_t alg_id;
    struct tkm_auth_info auth;
    uint8_t *iv;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *mac;
    uint32_t mac_len;
} __attribute__((packed));

struct symkey_encrypt_key_export_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth_enc;
    uint32_t key2;
    struct tkm_auth_info auth;
    uint32_t padding_mode;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct ecc_pubkey_encrypt_key_export_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t ecckey_type;
    ecc_ref_public_key *ecc_pub;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct rsa_pubkey_encrypt_key_export_request {
    int32_t return_code;
    uint32_t key;
    struct tkm_auth_info auth;
    uint32_t padding; // default: RSA_PKCS1_PADDING
    rsa_ref_public_key *rsa_pub;
    struct key_cipher *enc_key;
} __attribute__((packed));

struct ecc_sharedkey_gen_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t role;
    uint32_t key1;
    struct tkm_auth_info auth1;
    uint32_t key2;
    struct tkm_auth_info auth2;
    ecc_ref_public_key *ecc_pub1;
    ecc_ref_public_key *ecc_pub2;
    uint8_t *uid_self;
    uint32_t uid_self_len;
    uint8_t *uid_other;
    uint32_t uid_other_len;
    uint32_t handle;
    struct tkm_auth_info auth_setting;
    uint8_t *dgst_s1;
    uint32_t dgst_s1_len;
    uint8_t *dgst_sa;
    uint32_t dgst_sa_len;
} __attribute__((packed));

struct ecc_sharedkey_export_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t role;
    uint32_t key1;
    struct tkm_auth_info auth1;
    uint32_t key2;
    struct tkm_auth_info auth2;
    ecc_ref_public_key *ecc_pub1;
    ecc_ref_public_key *ecc_pub2;
    uint8_t *uid_self;
    uint32_t uid_self_len;
    uint8_t *uid_other;
    uint32_t uid_other_len;
    uint8_t *dgst_s1;
    uint32_t dgst_s1_len;
    uint8_t *dgst_sa;
    uint32_t dgst_sa_len;
    uint8_t *shared_key;
    uint32_t shared_len;
} __attribute__((packed));

struct encrypt_envelope_request {
    int32_t return_code;
    ecc_ref_public_key *ecc_pub;
    uint32_t alg_id;
    uint32_t key_bits;
    struct key_cipher *enc_key;
    uint32_t padding_mode;
    uint32_t crypt_mode;
    uint8_t *iv;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

struct sym_crypt_with_enckey_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t handle;
    struct tkm_auth_info auth;
    uint32_t key_padding;
    struct key_cipher *enc_key;
    uint32_t padding_mode;
    uint32_t crypt_mode;
    uint8_t *iv;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

struct sym_crypt_with_kdf_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t alg_id;
    uint32_t key_bits;
    uint32_t kdf_mode;
    kdf_param_t kdf_para;
    uint32_t padding_mode;
    uint32_t crypt_mode;
    uint8_t *iv;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

struct hmac_with_kdf_request {
    int32_t return_code;
    uint32_t flag;
    uint32_t alg_id;
    uint32_t key_bits;
    uint32_t kdf_mode;
    kdf_param_t kdf_para;
    uint32_t halg;
    uint8_t *src_data;
    uint32_t src_len;
    uint8_t *dest_data;
    uint32_t dest_len;
} __attribute__((packed));

EXPORT_API uint32_t get_psp_lib_version(void);
EXPORT_API int tkm_dev_init(void);
EXPORT_API int tkm_api_example(unsigned int testcode);
/* Management interfaces */
EXPORT_API int tkm_device_info_get(struct tkm_device_info *info);
EXPORT_API int tkm_key_info_get(struct key_info_request *reqs, uint32_t num);
EXPORT_API int tkm_key_gen(struct key_gen_request *reqs, uint32_t num);
EXPORT_API int tkm_key_refresh(struct key_refresh_request *reqs, uint32_t num);
EXPORT_API int tkm_key_image_load(unsigned char *image, unsigned int length);
EXPORT_API int tkm_key_image_save(unsigned char *image, unsigned int *length);
EXPORT_API int tkm_dev_pubkey_cert_export(unsigned int cert_type, unsigned char *key_cert, unsigned int *length);
EXPORT_API int tkm_change_sysauth(enum SYS_SEC_ROLE role, struct tkm_auth_info *old_auth, struct tkm_auth_info *new_auth);
EXPORT_API int tkm_device_config_set(struct tkm_auth_info *sysauth, uint8_t type, uint8_t *buf, uint32_t length);
EXPORT_API int tkm_device_config_show(uint8_t type, uint8_t *buf, uint32_t *length);
EXPORT_API int tkm_key_backup(struct tkm_auth_info *sysauth, uint32_t scheme, uint8_t *cert_chain, \
        uint32_t cert_chain_len, uint8_t *shared_data, uint32_t *shared_length, uint8_t *image, uint32_t *length)
;
EXPORT_API int tkm_key_restore(struct tkm_auth_info *sysauth, uint32_t scheme, uint8_t *shared_data, uint32_t shared_length, uint8_t *image, uint32_t length)
;

/* Business interfaces */
EXPORT_API int tkm_change_keyauth(struct change_keyauth_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_pubkey_export_batch(struct ecc_pubkey_export_request *reqs, uint32_t num);
EXPORT_API int tkm_sym_keygen_sym_encrypt_batch(struct sym_keygen_sym_encrypt_request *reqs, uint32_t num);
EXPORT_API int tkm_asym_keygen_sym_encrypt_batch(struct asym_keygen_sym_request *reqs, uint32_t num);
EXPORT_API int tkm_keyimport_decrypt_and_load_batch(struct keyimport_decrypt_and_load_request *reqs, uint32_t num);
EXPORT_API int tkm_kdf_batch(struct kdf_request *reqs, uint32_t num);
EXPORT_API int tkm_key_destroy_batch(struct key_destroy_request *reqs, uint32_t num);
EXPORT_API int tkm_symkey_encrypt_key_export_batch(struct symkey_encrypt_key_export_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_pubkey_encrypt_key_export_batch(struct ecc_pubkey_encrypt_key_export_request *reqs, uint32_t num);
EXPORT_API int tkm_rsa_pubkey_encrypt_key_export_batch(struct rsa_pubkey_encrypt_key_export_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_sign_batch(struct ecc_sign_request *reqs, uint32_t num);
EXPORT_API int tkm_rsa_sign_batch(struct rsa_sign_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_decrypt_batch(struct ecc_decrypt_request *reqs, uint32_t num);
EXPORT_API int tkm_rsa_decrypt_batch(struct rsa_decrypt_request *reqs, uint32_t num);
EXPORT_API int tkm_hmac_batch(struct hmac_request *reqs, uint32_t num);
EXPORT_API int tkm_cmac_batch(struct cmac_request *reqs, uint32_t num);
EXPORT_API int tkm_sym_crypt_batch(struct sym_crypt_request *reqs, uint32_t num);
EXPORT_API int tkm_rsa_pubkey_export_batch(struct rsa_pubkey_export_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_sharedkey_gen_batch(struct ecc_sharedkey_gen_request *reqs, uint32_t num);
EXPORT_API int tkm_ecc_sharedkey_export_batch(struct ecc_sharedkey_export_request *reqs, uint32_t num);
EXPORT_API int tkm_encrypt_envelope_with_pubkey_batch(struct encrypt_envelope_request *reqs, uint32_t num);
EXPORT_API int tkm_sym_crypt_with_enckey_batch(struct sym_crypt_with_enckey_request *reqs, uint32_t num);
EXPORT_API int tkm_sym_crypt_with_kdf_batch(struct sym_crypt_with_kdf_request *reqs, uint32_t num);
EXPORT_API int tkm_hmac_with_kdf_batch(struct hmac_with_kdf_request *reqs, uint32_t num);
#endif
