#ifndef _TKM_CERT_H
#define _TKM_CERT_H

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility ("default")))
#define LOCAL_API  __attribute__ ((visibility ("hidden")))
#endif

#define SERIAL_NUMBER_LEN 32
#define ISSUER_LEN 16
#define TIME_LEN 32
#define SUBJECT_LEN 128
#define CERTIFYING_ID_LEN 16
#define ECC_MAXLEN 72
#define RSA_MODULUS_MAXLEN 512
#define KEY_ID_LEN 16
#define USER_ID_LEN 256
#define HY_CERT_RESERVED1_LEN 24
#define HY_CERT_RESERVED2_LEN 108
#define HY_CERT_RESERVED3_LEN 112

#pragma pack(1)
typedef struct {
    unsigned int curve;
    unsigned char qx[ECC_MAXLEN];
    unsigned char qy[ECC_MAXLEN];
} pub_key_ecc;

typedef struct {
    unsigned int modulus_size;
    unsigned char pubexp[RSA_MODULUS_MAXLEN];
    unsigned char modulus[RSA_MODULUS_MAXLEN];
} pub_key_rsa;

typedef union {
    pub_key_ecc pke;
    pub_key_rsa pkr;
} pub_key_para;

typedef struct pubkey {
    unsigned int pubkey_alg;
    pub_key_para pk;
} pubkey_t;

typedef struct c_cert_header {
    unsigned int version;
    unsigned char serial_number[SERIAL_NUMBER_LEN];
    unsigned int sig_alg;
    unsigned char issuer[ISSUER_LEN];
    char not_before[TIME_LEN];
    char not_after[TIME_LEN];
    unsigned char subject[SUBJECT_LEN];
} c_cert_header_t;

typedef struct ecc_signature {
    unsigned char r[ECC_MAXLEN];
    unsigned char s[ECC_MAXLEN];
} ecc_sig_t;

/*customer cert*/
typedef struct tkm_c_cert {
    c_cert_header_t ch;
    pubkey_t pubkey;
    unsigned char certifying_id[CERTIFYING_ID_LEN];
    ecc_sig_t ec_sig;
} tkm_c_cert_t;

/*hygon cert*/
typedef struct tkm_hy_cert {
    unsigned int version;
    unsigned char key_id[KEY_ID_LEN];
    unsigned char certifying_id[CERTIFYING_ID_LEN];
    unsigned int key_usage;
    unsigned char reserved1[HY_CERT_RESERVED1_LEN];
    pub_key_ecc pke;
    unsigned char user_id[USER_ID_LEN];
    unsigned char reserved2[HY_CERT_RESERVED2_LEN];
    ecc_sig_t ec_sig;
    unsigned char reserved3[HY_CERT_RESERVED3_LEN];
} tkm_hy_cert_t;

typedef struct tkm_cert {
    tkm_hy_cert_t hy_cert;
    tkm_c_cert_t c_cert;
} tkm_cert_t;

//trusted platform infomation struct
struct tkm_tp_info {
    uint32_t version;
    uint32_t pcr_algo;
    uint32_t pcr_list;
    uint32_t reserve;
    uint64_t time; // real time
    uint8_t pcr_aggregate_exp[MAX_HASH_LEN];
};

struct tkm_tp_info_cert {
    struct tkm_tp_info tti;
    uint8_t sm2_sig[64];
};

#pragma pack()

/* Management interfaces */
EXPORT_API int tkm_tp_cert_import(struct tkm_auth_info *sysauth, struct tkm_tp_info_cert *cert);
#endif
